@@ -1,16 +1,16 @@
 package Net::DNS::Parameters;
 
 #
-# $Id: Parameters.pm 1501 2016-08-25 14:57:12Z willem $
+# $Id: Parameters.pm 1513 2016-11-12 03:20:58Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1501 $)[1];
+$VERSION = (qw$LastChangedRevision: 1513 $)[1];
 
 
 ################################################
 ##
 ##	Domain Name System (DNS) Parameters
-##	(last updated 2016-08-11)
+##	(last updated 2016-10-18)
 ##
 ################################################
 
@@ -19,6 +19,12 @@ use strict;
 use integer;
 use Carp;
 
+use constant DNSEXTLANG => defined eval {
+	local $SIG{__WARN__} = sub { };
+	new FileHandle("RRTYPEgen |");
+};
+
+
 use base qw(Exporter);
 use vars qw(@EXPORT);
 @EXPORT = qw(
@@ -99,7 +105,7 @@ use vars qw( %typebyname %typebyval );
 	NSEC3PARAM => 51,					# RFC5155
 	TLSA	   => 52,					# RFC6698
 	SMIMEA	   => 53,					# draft-ietf-dane-smime
-	HIP	   => 55,					# RFC-ietf-hip-rfc5205-bis-10
+	HIP	   => 55,					# RFC8005
 	NINFO	   => 56,					#
 	RKEY	   => 57,					#
 	TALINK	   => 58,					#
@@ -245,11 +251,14 @@ sub classbyval {
 sub typebyname {
 	my $name = shift;
 
-	$typebyname{$name} || $typebyname{uc $name} || do {
-		croak "unknown type $name" unless $name =~ m/(TYPE)?(\d+)/i;
-		my $val = 0 + $2;
-		croak "typebyname( $name ) out of range" if $val > 0xffff;
-		return $val;
+	$typebyname{$name} || do {
+		if ( $name =~ m/(TYPE)?(\d+)/i ) {
+			my $val = 0 + $2;
+			croak "typebyname( $name ) out of range" if $val > 0xffff;
+			return $val;
+		}
+		_typespec("$name.RRNAMES");
+		return $typebyname{uc $name} || croak "unknown type $name";
 			}
 }
 
@@ -259,7 +268,9 @@ sub typebyval {
 	$typebyval{$val} || do {
 		$val += 0;
 		croak "typebyval( $val ) out of range" if $val > 0xffff;
-		return "TYPE$val";
+		$typebyval{$val} = "TYPE$val";
+		_typespec("$val.RRTYPES");
+		return $typebyval{$val};
 			}
 }
 
@@ -303,6 +314,40 @@ sub ednsoptionbyval {
 }
 
 
+sub register {				## register( 'TOY', 1234 )	(NOT part of published API)
+	my ( $mnemonic, $rrtype ) = map uc($_), @_;		# uncoverable pod
+	$rrtype = rand(255) + 65280 unless $rrtype;
+	for ( typebyval $rrtype = int($rrtype) ) {
+		croak "'$mnemonic' is a CLASS identifier" if $classbyname{$mnemonic};
+		return $rrtype if /^$mnemonic$/;    # duplicate registration
+		croak "'$mnemonic' conflicts with TYPE$rrtype ($_)" unless /^TYPE\d+$/;
+		my $known = $typebyname{$mnemonic};
+		croak "'$mnemonic' conflicts with TYPE$known" if $known;
+	}
+	$typebyval{$rrtype} = $mnemonic;
+	return $typebyname{$mnemonic} = $rrtype;
+}
+
+
+sub _typespec {				## draft-levine-dnsextlang
+	eval <<'END' if DNSEXTLANG;
+	my ($node) = @_;
+	my $repository = 'ARPA.';
+	require Net::DNS::Resolver;
+	my $resolver = new Net::DNS::Resolver;
+	my $response = $resolver->send( "$node.$repository", 'TXT' );
+	my ( $tag, $language, @stanza );
+	foreach my $txt ( grep $_->type eq 'TXT', $response->answer ) {
+		( $tag, $language, @stanza ) = $txt->txtdata;
+		next unless $tag =~ /^RRTYPE=\d+$/;
+		register( split /[:\s]/, $stanza[0] );
+		last if $language =~ /^EN/i;
+	}
+	return @stanza;
+END
+}
+
+
 1;
 __END__
 
